#!/usr/bin/perl
# livewebarte.pl
# A script to dowload content from the site of ARTE channel,
# @ Juan Domingo, 2010. Juan.Domingo@uv.es
# Closely based on previous work from Carmelo Ingrao
# Version 1.1 (October 27, 2013) by Uli Zappe
#  - can extract more URL variants successfully
#    (specifically those with more than one ".mp4" in their path
#    and those with additional data after the last ".mp4")
#  - updated SWF player URL
#  - works with either wget or curl
# This script is distributed under the terms and conditions
# of the GPLv3 General Public License version 3 or,
# at your choice, any later version.
# A copy of GPLv3 can be found in http://www.gnu.org/copyleft/gpl.html
#
# Usage: 
# livewebarte.pl URL flv_file
# where URL is http://liveweb.arte.tv/fr/video/each...event_page...
# and flv_file is any name you want for the downloaded content.
use strict;
use warnings;

# Check if a program is in path, or accessible in the place
# Returns the way to call the program.
sub checkCmd {
 my $cmd=shift;
 my $output_option = "";
 $output_option = " -O" if $cmd eq "wget";
 if (system("which $cmd > /dev/null") == 0)
 {
  return("$cmd$output_option");
 }
 if ( -e "./$cmd" )
 {
  return("./$cmd$output_option");
 }
 # not successful so far - if we look for wget, let's try curl instead
 if ($cmd eq "wget")
 {
  $cmd="wget (or alternatively curl)";
  if (system("which curl > /dev/null") == 0)
  {
   return("curl -o");
  }
  if ( -e "./curl" )
  {
   return("./curl -o");
  }
 }
 die("Program $cmd is not installed or accessible. Please, install it to run this script.\n");
}

# Two arguments: URL of the web arte event page and file to save the data
($#ARGV == 1) || die("Usage: $0 URL_of_Arte_event_page flv_file_to_save\n");

my $downloadCmd=checkCmd("wget");
my $rtmpdumpCmd=checkCmd("rtmpdump");

# Use wget to get the source page and store it into a variable
my $sourcepage=`$downloadCmd - $ARGV[0] 2> /dev/null`;
my $eventnumber;

# Extract the event number. We look for new LwEvent('number_here') assuming that
# there must be at least one withespace between new and LwEvent and zero or
# more between lwEvent and the brackets and between ' and the number
# Arte might change this in the future. Check if something stops working.
$sourcepage =~ /new\s+LwEvent\s*\(\s*\'\s*([\d]+)\s*\'/ || die("Cannot find event number in URL. Either URL is wrong or string new LwEvent not found in it.\n");
$eventnumber=$1;

# The xml event info file is downloaded into a variable.
# Again, the file name and/or URL could be changed by Arte...
my $xmlfinal="http://arte.vo.llnwd.net/o21/liveweb/events/event-".$eventnumber.".xml";
my $xmlok=`$downloadCmd - $xmlfinal 2> /dev/null`;

# Let's extract the interesting part. The URL we are interested in is under the label
# <urlHd> and has the form:
# <urlHd>rtmp://arte.fcod.llnwd.net/a2306/o25/MP4:liveweb/stream/number[_date]_name[.mp4_quality].mp4[?some_additional_data]</urlHd>
# We are interested in the part between MP4 and (the last) .mp4
$xmlok =~ /<urlHd>.*(MP4:.+\.mp4).*<\/urlHd>/ || die("Cannot find urlHd in this xml file:\n$xmlfinal\n String <urlHd>...</urlHd> not found.\n");
my $mp4hdcut2=$1;

# uncomment the following line for URL debugging
#print "URL: $mp4hdcut2\n"; exit;

# Finally, let's build the command to download the flash stream
my $command="$rtmpdumpCmd -r rtmp://arte.fcod.llnwd.net:1935/a2306/o25 -a a2306/o25 -f LNX 10,0,45,2 -W http://download.liveweb.arte.tv/o21/liveweb/flash/player.swf -t rtmp://arte.fcod.llnwd.net:1935/a2306/o25 -p http://liveweb.arte.tv/ -o $ARGV[1] -y $mp4hdcut2";

# If the command fails, interesting contents are stored in files for debugging purposes.
if (system("$command > /dev/null") != 0)
{
 print("Execution of download command has failed.\n");
 open(F,"> xmlok.xml");
 print F $xmlok;
 close(F);
 open(F,"> rtmp_command");
 print F $command;
 close(F);
 print("Check the file xmlok.xml and rtmp_command for more information.\n");
 die;
}

